import { __assign, __decorate, __extends, __metadata } from "tslib";
import { classNamesFunction } from '@uifabric-vue/utilities';
import { Component, Prop } from 'vue-property-decorator';
import { DirectionalHint } from '../../common/DirectionalHint';
import { clickedOutside } from '../../utilities/clickedOutside';
import { getMaxHeight, positionCallout } from '../../utilities/positioning';
import BaseComponent from '../BaseComponent';
var getClassNames = classNamesFunction();
var CalloutContentBase = /** @class */ (function (_super) {
    __extends(CalloutContentBase, _super);
    function CalloutContentBase() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.internalKey = new Date();
        _this.positions = null;
        _this.maxHeight = 0;
        _this.blockResetHeight = false;
        _this.heightOffset = 0;
        _this.positionAttempts = 0;
        return _this;
    }
    CalloutContentBase.prototype.created = function () {
        window.addEventListener('click', this.onGlobalClick, true);
        window.addEventListener('scroll', this.onGlobalScroll, true);
    };
    CalloutContentBase.prototype.beforeDestroy = function () {
        window.removeEventListener('click', this.onGlobalClick, true);
        window.removeEventListener('scroll', this.onGlobalScroll, true);
    };
    CalloutContentBase.prototype.updated = function () {
        this.internalKey = new Date();
    };
    CalloutContentBase.prototype.mounted = function () {
        this.updatePosition();
    };
    Object.defineProperty(CalloutContentBase.prototype, "classNames", {
        get: function () {
            var _a = this, theme = _a.theme, className = _a.className, calloutMaxWidth = _a.calloutMaxWidth, calloutMaxHeight = _a.calloutMaxHeight, backgroundColor = _a.backgroundColor, finalHeight = _a.finalHeight, positions = _a.positions, styles = _a.styles, calloutWidth = _a.calloutWidth, beakWidth = _a.beakWidth;
            return getClassNames(styles, {
                theme: theme,
                className: className,
                overflowYHidden: !!finalHeight,
                calloutWidth: calloutWidth,
                positions: positions,
                beakWidth: beakWidth,
                backgroundColor: backgroundColor,
                calloutMaxWidth: calloutMaxWidth,
            });
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(CalloutContentBase.prototype, "actualBeakWidth", {
        get: function () {
            return Math.sqrt(this.beakWidth * this.beakWidth * 2);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(CalloutContentBase.prototype, "positionCss", {
        get: function () {
            if (!this.positions)
                return null;
            var positionCss = this.positions;
            for (var key in positionCss.elementPosition) {
                positionCss.elementPosition[key] = "".concat(positionCss.elementPosition[key], "px");
            }
            for (var key in positionCss.beakPosition.elementPosition) {
                positionCss.beakPosition.elementPosition[key] = "".concat(positionCss.beakPosition.elementPosition[key], "px");
            }
            return positionCss;
        },
        enumerable: false,
        configurable: true
    });
    CalloutContentBase.prototype.updatePosition = function () {
        var positions = this.positions;
        var currentProps = __assign({}, this.$props);
        currentProps.bounds = this.bounds;
        currentProps.target = this.target;
        var newPositions = positionCallout(currentProps, this.$refs.hostElement, this.$refs.calloutElement, this.positions);
        if ((!positions && newPositions) ||
            (positions && newPositions && !this._arePositionsEqual(positions, newPositions) && this.positionAttempts < 5)) {
            this.positionAttempts++;
            this.positions = newPositions;
        }
        else if (this.positionAttempts > 0) {
            this.positionAttempts = 0;
            this.$emit('positioned', this.positions);
        }
    };
    CalloutContentBase.prototype.onGlobalClick = function (e) {
        var outside = clickedOutside(e, this.$refs.calloutElement);
        if (outside)
            this.$emit('dismiss', e);
    };
    CalloutContentBase.prototype.onGlobalScroll = function (e) {
        this.$emit('dismiss', e);
    };
    Object.defineProperty(CalloutContentBase.prototype, "bounds", {
        get: function () {
            return {
                top: 0 + this.minPagePadding,
                left: 0 + this.minPagePadding,
                right: window.innerWidth - this.minPagePadding,
                bottom: window.innerHeight - this.minPagePadding,
                width: window.innerWidth - this.minPagePadding * 2,
                height: window.innerHeight - this.minPagePadding * 2,
            };
        },
        enumerable: false,
        configurable: true
    });
    // Max height should remain as synchronous as possible, which is why it is not done using set state.
    // It needs to be synchronous since it will impact the ultimate position of the callout.
    CalloutContentBase.prototype._getMaxHeight = function () {
        var _this = this;
        if (!this.maxHeight) {
            if (this.directionalHintFixed && this.target) {
                var beakWidth = this.isBeakVisible ? this.beakWidth : 0;
                var gapSpace = this.gapSpace ? this.gapSpace : 0;
                // Since the callout cannot measure it's border size it must be taken into account here. Otherwise it will
                // overlap with the target.
                var totalGap_1 = gapSpace + beakWidth;
                this._async.requestAnimationFrame(function () {
                    if (_this.target) {
                        _this.maxHeight = getMaxHeight(_this.target, _this.directionalHint, totalGap_1, _this.bounds, _this.coverTarget);
                        _this.blockResetHeight = true;
                        _this.$forceUpdate();
                    }
                }, this.target);
            }
            else {
                this.maxHeight = this.bounds.height;
            }
        }
        return this.maxHeight;
    };
    CalloutContentBase.prototype._arePositionsEqual = function (positions, newPosition) {
        return (this._comparePositions(positions.elementPosition, newPosition.elementPosition) &&
            this._comparePositions(positions.beakPosition.elementPosition, newPosition.beakPosition.elementPosition));
    };
    CalloutContentBase.prototype._comparePositions = function (oldPositions, newPositions) {
        for (var key in newPositions) {
            // This needs to be checked here and below because there is a linting error if for in does not immediately have an if statement
            if (newPositions.hasOwnProperty(key)) {
                var oldPositionEdge = oldPositions[key];
                var newPositionEdge = newPositions[key];
                if (oldPositionEdge !== undefined && newPositionEdge !== undefined) {
                    if (oldPositionEdge.toFixed(2) !== newPositionEdge.toFixed(2)) {
                        return false;
                    }
                }
                else {
                    return false;
                }
            }
        }
        return true;
    };
    CalloutContentBase.prototype.render = function (h) {
        var _a = this, classNames = _a.classNames, positionCss = _a.positionCss, calloutMaxHeight = _a.calloutMaxHeight, overflowYHidden = _a.overflowYHidden, isBeakVisible = _a.isBeakVisible;
        var getContentMaxHeight = this._getMaxHeight()
            ? this._getMaxHeight() + this.heightOffset
            : undefined;
        var contentMaxHeight = calloutMaxHeight && getContentMaxHeight && calloutMaxHeight < getContentMaxHeight
            ? calloutMaxHeight
            : getContentMaxHeight;
        var overflowStyle = __assign({ 
            // ...this.$attrs.style,
            maxHeight: contentMaxHeight }, (overflowYHidden && { overflowY: 'hidden' }));
        return h('div', {
            ref: 'hostElement',
            class: classNames.container,
        }, [
            h('div', {
                ref: 'calloutElement',
                class: classNames.root,
                style: positionCss ? positionCss.elementPosition : null,
            }, [
                isBeakVisible && h('div', {
                    class: classNames.beak,
                    style: (positionCss && positionCss.beakPosition) ? positionCss.beakPosition.elementPosition : null,
                }),
                isBeakVisible && h('div', { class: classNames.beakCurtain }),
                h('div', { class: classNames.calloutMain, style: overflowStyle }, [
                    this.$slots.default,
                ]),
            ]),
        ]);
    };
    __decorate([
        Prop({ type: HTMLElement, required: true }),
        __metadata("design:type", HTMLElement)
    ], CalloutContentBase.prototype, "target", void 0);
    __decorate([
        Prop({ type: Number, default: null }),
        __metadata("design:type", Number)
    ], CalloutContentBase.prototype, "calloutWidth", void 0);
    __decorate([
        Prop({ type: Number, default: 16 }),
        __metadata("design:type", Number)
    ], CalloutContentBase.prototype, "beakWidth", void 0);
    __decorate([
        Prop({ type: Number, default: 8 }),
        __metadata("design:type", Number)
    ], CalloutContentBase.prototype, "minPagePadding", void 0);
    __decorate([
        Prop({ type: Boolean, default: true }),
        __metadata("design:type", Boolean)
    ], CalloutContentBase.prototype, "isBeakVisible", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], CalloutContentBase.prototype, "coverTarget", void 0);
    __decorate([
        Prop({ type: Number, default: 0 }),
        __metadata("design:type", Number)
    ], CalloutContentBase.prototype, "gapSpace", void 0);
    __decorate([
        Prop({ type: Number, default: undefined }),
        __metadata("design:type", Number)
    ], CalloutContentBase.prototype, "finalHeight", void 0);
    __decorate([
        Prop({ type: Number, default: undefined }),
        __metadata("design:type", Number)
    ], CalloutContentBase.prototype, "calloutMaxWidth", void 0);
    __decorate([
        Prop({ type: Number, default: undefined }),
        __metadata("design:type", Number)
    ], CalloutContentBase.prototype, "calloutMaxHeight", void 0);
    __decorate([
        Prop({ type: String, default: undefined }),
        __metadata("design:type", String)
    ], CalloutContentBase.prototype, "backgroundColor", void 0);
    __decorate([
        Prop({ type: Number, default: DirectionalHint.bottomAutoEdge }),
        __metadata("design:type", Number)
    ], CalloutContentBase.prototype, "directionalHint", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], CalloutContentBase.prototype, "directionalHintFixed", void 0);
    __decorate([
        Prop({ type: Boolean, default: false }),
        __metadata("design:type", Boolean)
    ], CalloutContentBase.prototype, "overflowYHidden", void 0);
    CalloutContentBase = __decorate([
        Component
    ], CalloutContentBase);
    return CalloutContentBase;
}(BaseComponent));
export { CalloutContentBase };
//# sourceMappingURL=CalloutContent.base.js.map